import { BoxCollider, Button, Collider, Component, ConeCollider, CylinderCollider, ITriggerEvent, Label, Node, NodeEventType, SphereCollider, Vec3, _decorator, find, game } from 'cc';
import { StormSunderAudioMgr } from '../Manager/StormSunderAudioMgr';
import { EventDispatcher } from 'db://assets/core_tgx/easy_ui_framework/EventDispatcher';
import { GameEvent } from '../Enum/GameEvent';
import { UIJoyStick } from '../UIJoyStick';
import { GameMgr, GameStatus } from '../Manager/GameMgr';
import { PropComponent, PropStatus } from './PropComponent';
const { ccclass, property } = _decorator;

const rotateSpeed = 500;

/** 龙卷风组件*/
@ccclass('TornadoComponent')
export class TornadoComponent extends Component {
    @property({ type: [Node] })
    points: Node[] = [];

    tornado: Node = null!;
    tigger: Collider = null!; //龙卷风触发器
    radiusTigger: Collider = null!;     //龙卷风半径检测触发器

    attack: number = 20;
    speed: number = 50;
    hp: number = 100;
    ai: boolean = false;

    _attackInterval: number = 0.5; // 攻击间隔
    _lastAttackTime: Map<string, number> = new Map(); // 记录上次攻击时间

    protected start(): void {
        this.initilizeUI();
        this.registerEvent();
    }

    initilizeUI(): void {
        if (!this.node) return;

        this.tornado = this.node.getChildByName('tornado')!;
        this.tigger = this.tornado.getComponent(Collider)!;

        this.tigger.on('onTriggerEnter', this.onTriggerEnter, this);
        this.tigger.on('onTriggerStay', this.onTriggerStay, this);
        this.tigger.on('onTriggerExit', this.onTriggerExit, this);
    }

    onTriggerEnter(event: ITriggerEvent): void {

    }

    onTriggerStay(event: ITriggerEvent): void {
        if (event.otherCollider.getGroup() == 1 << 4) {
            const otherCollider = event.otherCollider;
            const propComp = otherCollider.node.getComponent(PropComponent);

            if (!propComp) return;

            const currentTime = game.totalTime;
            const nodeId = otherCollider.node.uuid;
            const lastTime = this._lastAttackTime.get(nodeId) || 0;

            // 检查是否达到攻击间隔
            if (currentTime - lastTime >= this._attackInterval * 1000) {
                this._lastAttackTime.set(nodeId, currentTime);

                if (propComp.status == PropStatus.DIE) return
                // 造成伤害
                propComp.hurt(this.attack);

                // 检查道具是否被摧毁
                if (propComp.hp <= 0) {
                    // 随机选择一个吸收点
                    const randomPoint = this.points[Math.floor(Math.random() * this.points.length)];
                    if (randomPoint) {
                        otherCollider.node.parent = randomPoint;
                        otherCollider.node.setPosition(Vec3.ZERO);
                        propComp.swallow();
                    }
                }
            }
        }
    }

    onTriggerExit(event: ITriggerEvent): void {

    }

    update(deltaTime: number) {
        if (GameMgr.inst.gameStatus != GameStatus.Playing) return;
        this.MoveHandler();
        // //自身旋转
        this.RotateHandler();
    }

    RotateHandler(): void {
        this.node.eulerAngles = new Vec3(this.node.eulerAngles.x, this.node.eulerAngles.y + rotateSpeed * game.deltaTime, this.node.eulerAngles.z);
        // console.log('this.node.eulerAngles:', this.node.eulerAngles);
    }

    MoveHandler(): void {
        const playerDir = UIJoyStick.ins.dir;
        const playerX = playerDir.x * this.speed * game.deltaTime;
        const playerZ = playerDir.y * this.speed * game.deltaTime;

        this.node.setPosition(this.node.position.x + playerX, 0, this.node.position.z - playerZ);
    }


    protected registerEvent() {
        EventDispatcher.instance.on(GameEvent.EVENT_STORM_LEVEL_UP, this.stormLevelUp, this);
    }

    private stormLevelUp() {
        //DOTO  计算攻击速度等属性
        this.attack = 1;
        this.speed = 1;

        //DOTO 更新UI
    }

    protected onDestroy(): void {
        this.unregisterEvent();
    }

    protected unregisterEvent() {
        EventDispatcher.instance.off(GameEvent.EVENT_STORM_LEVEL_UP, this.stormLevelUp, this);
    }

}
